
"""
Convert COSMO-GHG output from SMARTCARB-2 to simple Level-2 dataset
by extracting all required data and binning 2x2 grid cells.
"""


from collections import defaultdict

import xarray

import amrs
import smartcarb
import smartcarb.clouds


INPUT_PATH = '/bigdat/kug/smartcarb/smartcarbExtBEL/'
OUTPUT_PATH = '/output/SMARTCARB/output/mass-balance/PointSources.Ext'


def read_xco2_field(filename):
    
    path = os.path.dirname(filename)
    
    constant_filename = os.path.join(path, 'lffd20180606000000c.nc')
    with xarray.open_dataset(constant_filename) as d:
        rlon = d.rlon.values[:]
        rlat = d.rlat.values[:]
        lon = d.lon.values[:]
        lat = d.lat.values[:]
        HHL = d.HHL.values.copy()
    
    with xarray.open_dataset(filename) as d:
        P = d.P.values[:]
        PS = d.PS.values[:]
        QV = d.QV.values[:]
        
        U = d.U.values[:]    
        V = d.V.values[:]            
        
        CLCT = d.CLCT.values[:]
        
        CO2_BG = (d.CO2_BG + d.CO2_A + d.CO2_O + d.CO2_RA - d.CO2_GPP).values[:]   
        
        CO2_A = d.CO2_BELA.values[:]
        CO2_B = d.CO2_BELB.values[:]
        CO2_C = d.CO2_BELC.values[:]        
        
    return lon, lat, rlon, rlat, HHL, P, PS, QV, CLCT, CO2_BG, CO2_A, CO2_B, CO2_C, U, V



def generate_satellite_data(filenames):

    # create defaultdict
    values = defaultdict(list)

    for filename in filenames:
        print(filename)

        time = pandas.Timestamp(datetime.strptime(os.path.basename(filename), 'lffd%Y%m%d%H%M%S.nc') )
        values['time'].append( time.to_numpy() )

        lon, lat, rlon, rlat, HHL, P, PS, QV, CLCT, CO2_BG, CO2_A, CO2_B, CO2_C, U, V = read_xco2_field(filename)

        mair = amrs.misc.chem.calculate_mair(p=P, ps=PS, h=HHL, q=QV)
        
        # CO2
        xco2_BG = amrs.misc.chem.calculate_xco2(CO2_BG, mair, q=0.0)
        xco2_A = amrs.misc.chem.calculate_xco2(CO2_A, mair, q=0.0)
        xco2_B = amrs.misc.chem.calculate_xco2(CO2_B, mair, q=0.0)
        xco2_C = amrs.misc.chem.calculate_xco2(CO2_C, mair, q=0.0)

        for name, value in [
            ('lon', lon),
            ('lat', lat),
            ('HHL', HHL[0]),
            ('CO2_A', CO2_A[0]),
            ('CO2_B', CO2_B[0]),
            ('CO2_C', CO2_C[0]),
            ('CO2_BG', CO2_BG[0]),
            ('XCO2_A', xco2_A),
            ('XCO2_B', xco2_B),
            ('XCO2_C', xco2_C),
            ('XCO2_BG', xco2_BG),
            ('psurf', PS[0]),
            ('clouds', CLCT[0]),
            ('U', U[0]),
            ('V', V[0]),

        ]:
            #print(name, np.shape(value))
            if np.ndim(value) == 2:
                value = spatial_binning(value, 2, 2)
            else:
                value = spatial_binning(value.swapaxes(0,1).swapaxes(1,2), 2, 2)
            
            values[name].append(value)

    values = dict((k, np.array(v)) for k, v in values.items())
    
    data = xarray.Dataset()

    data['time'] = xarray.DataArray(values['time'], dims='time')
    data.attrs['orbit'] = 0
    data.attrs['lon_eq'] = 0

    values['CO2_std'] = 0.5 * np.ones_like(values['CO2_BG'])

    for name in sorted(values.keys()):
        
        if name == 'time':
            continue
        
        if np.ndim(values[name]) == 3:
            data[name] = xarray.DataArray(values[name], dims=('time', 'nobs', 'nrows'))
        
        else:
            if np.shape(values[name])[-1] == 61:
                data[name] = xarray.DataArray(values[name], dims=('time', 'nobs', 'nrows', 'nlevel1'))                
            else:
                data[name] = xarray.DataArray(values[name], dims=('time', 'nobs', 'nrows', 'nlevel'))            
        
        if name.startswith('CO2'):
            data[name].attrs['units'] = 'ppm'

    # add pixel corners
    lonc_and_latc = np.array([smartcarb.clouds.compute_pixel_boundaries(data['lon'][i], data['lat'][i]) for i in range(data.time.size)])
    lonc = lonc_and_latc[:,0,:,:,:]
    latc = lonc_and_latc[:,1,:,:,:]

    data['lonc'] = xarray.DataArray(lonc, dims=('time', 'nobs', 'nrows', 'ncorner'))
    data['latc'] = xarray.DataArray(latc, dims=('time', 'nobs', 'nrows', 'ncorner'))
    
    
    return data


if __name__ == '__main__':
    filenames = sys.args[1:]
    
    print(filenames)
    
    data = generate_satellite_data(filenames)
    
    # TODO: compression
    output_filename = 'test.nc'
    data.to_netcdf(output_filename)
    